package gift

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/logic/gift_collect"
	"live-service/app/user_center/rpc/internal/logic/integral"
	"live-service/app/user_center/rpc/internal/logic/platform_user"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"strconv"

	"github.com/zeromicro/go-zero/core/logx"
)

type UserSendGiftLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUserSendGiftLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserSendGiftLogic {
	return &UserSendGiftLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// UserSendGift 用户赠送礼物
// 1. 记录用户赠送信息
// 2. 新增用户积分
func (l *UserSendGiftLogic) UserSendGift(in *pb.UserSendGiftReq) (*pb.UserSendGiftResp, error) {
	// 首先获取用户id
	var err error
	sysUser, err := platform_user.NewRetrievePlatformUserLogic(l.ctx, l.svcCtx).RetrievePlatformUser(&pb.PlatformUserReq{
		Platform: in.Platform,
		PUid:     in.PUid,
	})
	if err != nil {
		return nil, err
	}

	resp := &pb.UserSendGiftResp{
		User: sysUser.User,
	}

	// 记录送礼信息
	{
		dbUserGift := &model.UserGift{
			Id:       uuid.NextId(),
			UserId:   sysUser.User.Id,
			Platform: in.Platform,
			RoomId:   in.RoomId,
			GiftId:   strconv.FormatInt(in.GiftId, 10),
			GiftName: in.GiftName,
			Num:      in.Num,
		}
		if in.IsPaid {
			dbUserGift.Price = in.Price
		} else {
			dbUserGift.FreePrice = in.Price
		}
		if err = l.svcCtx.UserGiftModel.Insert(l.ctx, dbUserGift); err != nil {
			l.Logger.Errorf("记录用户[%d]送礼信息 [%d:%s:%d] 失败，操作继续...", sysUser.User.Id, in.GiftId, in.GiftName, in.Num)
		}
	}

	// 积分
	{
		// 待增加的积分
		addonIntegral := calcIntegral(l.svcCtx.Config, in.Platform, in.IsPaid, float64(in.Price), in.Num)
		if tmpMap, ok := gift_collect.Service.GetCacheByPlatform(in.Platform); ok {
			if tmpData, ok := tmpMap[in.GiftId]; ok {
				addonIntegral = calcIntegral(l.svcCtx.Config, in.Platform, tmpData.IsPaid, tmpData.Price, in.Num)
			}
		}

		newIntegral, err := integral.NewChangeIntegralLogic(l.ctx, l.svcCtx).ChangeIntegral(&pb.ChangeIntegralReq{
			UserId: sysUser.User.Id,
			Change: addonIntegral,
		})
		if err != nil {
			return nil, err
		}
		resp.Integral = newIntegral
	}

	return resp, nil
}
