package integral

import (
	"context"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type ChangeIntegralLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewChangeIntegralLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ChangeIntegralLogic {
	return &ChangeIntegralLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// ChangeIntegral 新增用户积分
func (l *ChangeIntegralLogic) ChangeIntegral(in *pb.ChangeIntegralReq) (*pb.ChangeIntegralResp, error) {
	if err := l.svcCtx.UserIntegralModel.Transact(l.ctx, nil, func(tx *gorm.DB) error {
		integral, err := l.svcCtx.UserIntegralModel.FindIntegral(l.ctx, tx, in.UserId)
		if err != nil {
			if errors.Is(err, model.ErrNotFound) {
				if in.Change < 0 {
					return errors.New("用户积分不足")
				}
				// 用户积分记录不存在,进行插入
				if err = l.svcCtx.UserIntegralModel.InsertTx(l.ctx, tx, &model.UserIntegral{
					UserId:   in.UserId,
					Integral: in.Change,
				}); err != nil {
					return errors.Wrap(err, "插入用户积分失败")
				}
				return nil
			} else {
				return errors.Wrap(err, "获取当前用户积分失败")
			}
		}
		if integral+in.Change < 0 {
			return errors.New("用户积分不足")
		}

		if err = l.svcCtx.UserIntegralModel.UpdateIntegralTx(l.ctx, tx, in.UserId, integral+in.Change); err != nil {
			return errors.Wrap(err, "更新用户积分失败")
		}
		return nil
	}); err != nil {
		return nil, errors.Wrapf(nerr.NewWithErr(err), "记录积分-事务执行失败, err:%+v", err)
	}

	// 查询当前用户积分
	integral, err := l.svcCtx.UserIntegralModel.FindIntegral(l.ctx, nil, in.UserId)
	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithMsg("查询用户积分失败"), "查询用户积分失败, err:%+v", err)
	}

	return &pb.ChangeIntegralResp{
		UserId:   in.UserId,
		Change:   in.Change,
		Integral: integral,
	}, nil
}
